package io.data2viz.charts.chart

import io.data2viz.charts.chart.mark.TooltipPosition
import io.data2viz.charts.core.*
import io.data2viz.charts.layout.DrawingZone
import io.data2viz.viz.RichTextBuilder

/**
 * The [Chart] tooltip visual component.
 */
public interface Tooltip<DOMAIN> {

    /**
     * Set the [Datum] formatter, by default, check if the Datum.domain object implements the [Labelled] interface and
     * displays Datum.domain.label, else the formatter will try to display infos based on the dimension used.
     *
     * @see [Labelled]
     */
    public var formatter: (Datum<DOMAIN>.() -> String)?


    /**
     * Define a "tooltip builder", a lambda that takes several parameters:
     *
     * - markIndex: the 0-based index of the mark on which the chart will display the tooltip
     * - datum: the [Datum] object on which the tooltip "applies"
     * - ttp: a (nullable) [TooltipPosition] storing where to place the tooltip on the [DrawingZone]
     * - title: a nullable [String], the title of your tooltip
     * - text: a nullable [String], the default text content of your tooltip
     * - drawingZone: the [DrawingZone] on which to draw the custom tooltip
     *
     * This lambda should return a [RichTextBuilder] (or null), then the tooltip itself will size and
     * display accordingly.
     */
    public var tooltipBuilder: (markIndex: Int, datum: Datum<DOMAIN>?, ttp: TooltipPosition?, title: String?, text: String?, drawingZone: DrawingZone) -> RichTextBuilder?

}

