package io.data2viz.charts.event

import io.data2viz.charts.chart.Chart
import io.data2viz.charts.chart.mark.RelativePoint
import io.data2viz.charts.core.Datum
import io.data2viz.charts.core.SelectedDatum
import io.data2viz.geom.Rect

/**
 * EventContext contains the event type and the event context (elements that are impacted, ratios, size, ...)
 */
public interface EventContext<DOMAIN> {

    /**
     * The event type
     */
    public val chart: Chart<DOMAIN>

    /**
     * The event type
     */
    public val event: UserEvent

    /**
     * A map of all Datums that are impacted by this event, sorted by their zero-based mark indexes.
     */
    public val selectedData: List<SelectedDatum<DOMAIN>>

    /**
     * An on-screen position coordinates in percentage, relative to the charting zone.
     */
    public val currentPosition: RelativePoint

    /**
     * The movement (if any) from the previous event to this one in relative coordinates
     */
    public val relativeMovement: RelativePoint

    /**
     * The zoom ratio (if any)
     */
    public val zoomAction: ZoomAction

    /**
     * An on-screen zone rectangle coordinates representing the current dragging if any, relative to the charting zone.
     */
    public val dragZone: Rect?

    /**
     * A list of Datums without the marks information
     */
    public val data: List<Datum<DOMAIN>>
}
