package io.data2viz.charts.event

import io.data2viz.charts.chart.Chart
import io.data2viz.charts.chart.mark.RelativePoint
import io.data2viz.charts.core.Datum
import io.data2viz.charts.core.SelectedDatum
import io.data2viz.charts.event.internal.EventController
import io.data2viz.geom.Rect

internal class EventContextImpl<DOMAIN>(
    eventController: EventController<DOMAIN>,
    override val event: UserEvent,
    override val relativeMovement: RelativePoint,
    override val dragZone: Rect?,
    wheelZoomAction: ZoomAction?
) : EventContext<DOMAIN> {

    override val chart: Chart<DOMAIN> by lazy { eventController.chart }

    override val currentPosition: RelativePoint by lazy { eventController.positionToRelativePoint(event.pos) }

    override val selectedData: List<SelectedDatum<DOMAIN>> by lazy {
        eventController.getIndexedData(event.pos, dragZone)
    }

    /**
     * If there is a specific wheel movement, then take this as the zoom action, else define a zoom action from other
     * parameters.
     */
    override val zoomAction: ZoomAction by lazy {
        wheelZoomAction ?: eventController.getZoomActionFromDragZone(currentPosition, dragZone)
    }

    override val data: List<Datum<DOMAIN>> by lazy { selectedData.map { it.datum }.toSet().toList() }
}
