package io.data2viz.charts.event

/**
 * An [EventListener] can listen and handle to [ChartEvent]s.
 */
public interface EventListener<DOMAIN> {

    /**
     * Add a listener on a [HighlightEvent].
     *
     * @param listener A lambda that takes a [HighlightEvent] and will be called each time a [HighlightEvent] occurs.
     */
    public fun onHighlight(listener: (event: HighlightEvent<DOMAIN>) -> Unit)

    /**
     * Add a listener on a [SelectEvent].
     *
     * @param listener A lambda that takes a [SelectEvent] and will be called each time a [SelectEvent] occurs.
     */
    public fun onSelect(listener: (event: SelectEvent<DOMAIN>) -> Unit)

    /**
     * Add a listener on a [SelectionAddEvent].
     *
     * @param listener A lambda that takes a [SelectionAddEvent] and will be called
     * each time a [SelectionAddEvent] occurs.
     */
    public fun onSelectionAdd(listener: (event: SelectionAddEvent<DOMAIN>) -> Unit)



    /**
     * Add a listener on a [SelectionRemoveEvent].
     *
     * @param listener A lambda that takes a [SelectionRemoveEvent] and will be called
     * each time a [SelectionRemoveEvent] occurs.
     */
    public fun onSelectionRemove(listener: (event: SelectionRemoveEvent<DOMAIN>) -> Unit)

    /**
     * Add a listener on a [ZoomEvent].
     *
     * @param listener A lambda that takes a [ZoomEvent] and will be called each time a [ZoomEvent] occurs.
     */
    public fun onZoom(listener: (event: ZoomEvent) -> Unit)

    /**
     * Add a listener on a [PanEvent].
     *
     * @param listener A lambda that takes a [PanEvent] and will be called each time a [PanEvent] occurs.
     */
    public fun onPan(listener: (event: PanEvent) -> Unit)

    /**
     * Push a [ChartEvent] to a chart.
     */
    public fun pushEvent(event: ChartEvent)
}
