package io.data2viz.charts.event

import io.data2viz.charts.chart.internal.NO_MARK_SELECTION_KEY
import io.data2viz.charts.core.Datum
import io.data2viz.charts.core.SelectedDatum

/**
 * A "selection event" is generally triggered when a user interacts with a [Mark] by clicking on it.
 *
 * @param selectedData the map of [Datum] to select for each markIndex
 * @param data the list of [Datum] to select.
 */
public class SelectEvent<DOMAIN> public constructor(
    public val selectedData: List<SelectedDatum<DOMAIN>>,
) : ChartEvent {

    public constructor(data: Collection<Datum<DOMAIN>>) : this(data.map { SelectedDatum(it, it.indexOfSeries, 0) })

    override var preventDefault: Boolean = false

    public val data: List<Datum<DOMAIN>> by lazy { selectedData.map { it.datum }.toSet().toList() }
}
