package io.data2viz.charts.event

import io.data2viz.geom.Point
import io.data2viz.viz.HasMetaKeys
import io.data2viz.viz.KMouseEvent
import io.data2viz.viz.KZoomEvent

/**
 * A ChartMouseEvent is a class storing the basic Data2Viz [KMouseEvent] and the [EventType].
 */
public data class UserEvent internal constructor(

    /**
     * The mouse event position
     */
    public val pos: Point,

    /**
     * The mouse event type
     */
    public val type: EventType,

    /**
     * Is the "meta" key pressed during event?
     */
    public val metaKey: Boolean = false,

    /**
     * Is the "alt" key pressed during event?
     */
    public val altKey: Boolean = false,

    /**
     * Is the "ctrl" key pressed during event?
     */
    public val ctrlKey: Boolean = false,

    /**
     * Is the "shift" key pressed during event?
     */
    public val shiftKey: Boolean = false
) {
    internal constructor(baseEvent: KMouseEvent, type: EventType) : this(
        baseEvent.pos,
        type,
        baseEvent.metaKey,
        baseEvent.altKey,
        baseEvent.ctrlKey,
        baseEvent.shiftKey
    )

    internal constructor(pos: Point, type: EventType, metaKeys: HasMetaKeys? = null) : this(
        pos,
        type,
        metaKeys?.metaKey ?: false,
        metaKeys?.altKey ?: false,
        metaKeys?.ctrlKey ?: false,
        metaKeys?.shiftKey ?: false
    )

    internal constructor(baseEvent: KZoomEvent, type: EventType) : this(
        baseEvent.startZoomPos,
        type,
        baseEvent.metaKey,
        baseEvent.altKey,
        baseEvent.ctrlKey,
        baseEvent.shiftKey
    )

    /**
     * No effective key is pressed (!metaKey && !altKey && !ctrlKey && !shiftKey)
     */
    public val noKey: Boolean
        get() = !metaKey && !altKey && !ctrlKey && !shiftKey
}
